/***************************************************
 * Display Board
 * Team Smart Tow
 * ************************************************/
        
#include <system.h>

#include "EESD.h"

#pragma DATA _CONFIG1H, _OSC_HSPLL_1H //40 mhz
#pragma DATA _CONFIG2H, _WDT_OFF_2H 
#pragma DATA _CONFIG4L, _LVP_OFF_4L 
#pragma DATA _CONFIG3H, _MCLRE_ON_3H
#pragma DATA _CONFIG2L, _BOREN_OFF_2L

#pragma CLOCK_FREQ 40000000

//defining global variables 
bool up_arrow = 1;
bool down_arrow = 1;
bool enter = 1;
bool cancel = 1;
short received = 0;
short byte_received = 0;
short identifier = 0b00000000;
short data = 0;
short LCD_state = 0;
short temp_alarm_armed = 1;
bool send_acknowledge = false;
bool door_cancel = false;
short loop_count = 0;

void main(void) {

  short temp_in = 60;
  short temp_out = 70;
  short temp_in_low_alarm = 35;
  short temp_in_high_alarm = 100;
  short temp_in_high_alarm2 = 100;
  short temp_in_low_alarm2 = 35;
  short backup_range = 20;
  short inches = 0;
  short feet = 0;
  short inches_after_feet = 0;
  bool too_close = 0;
  short true_false = 0;

  LCD_init();
  LCD_setpos(0,0);
  LCD_printf("Initializing...");
  LED_buzzer_init();
  interrupt_init(); //enable interrupt on change for B0 and B4-B7
  zigbee_init();
  smart_tow_start();  //flash some lights - make it look good
  LCD_clear();
 
    
  while(1) {
    
    loop_count = loop_count + 1;
    
    
    if(loop_count > 100) {   //Threshold for indicating loss of comm - could have been done with WDT too.
      LCD_clear();
      LCD_setpos(0,0);
      LCD_printf("Lost Comm with Trailer  ");
      LCD_setpos(1,0);
      LCD_printf(" Reset Trailer Board");
      while(loop_count > 100) { 
          
          lata.1 = 1;  //Right Blind
          lata.3 = 1;  //Left Blink
          lata.0 = 0;  //left blind
          lata.4 = 0;  //too close
          lata.5 = 0;  //right blink
          latc.3 = 0;  //icy
          lata.2 = 0;  //door
          
          delay_ms(500);
          
          lata.1 = 0;  //Right Blind
          lata.3 = 0;  //Left Blink
          lata.0 = 1;  //left blind
          lata.4 = 0;  //too close
          lata.5 = 1;  //right blink
          latc.3 = 0;  //icy
          lata.2 = 0;  //door          
          
          delay_ms(500);
      }
    }
    
    
    if(send_acknowledge) {  //received all 3 bytes
      while(!txsta) { }  //sends acknowledge to trailer
      txreg = 0b00000001;
      
      send_acknowledge = 0;  //resets send_acknowledge
           
      /*LCD_setpos(0,0);   //debugging
      LCD_bin(identifier);
      LCD_setpos(1,0);      
      LCD_bin(data);*/
      
      /* Figure out what data is and how to use */
      
      if(identifier == 0b00000000) { //true_false identifier
             
      true_false = data;
      if(true_false.4 == 0)  {   //if true_false.4 = 1, data is garbage
        if(true_false.0 == 0) {  //door closed 
          lata.2 = 0;  //Door LEDs off
          door_cancel = false;
          latc.4 = 0;  //turn buzzer off
          if(LCD_state == 3) {  //If door is low, make sure not in state 3.
            LCD_state = 0;  //back to default screen
          }
        }  
        if(true_false.0 == 1) {  //door open
          if(!door_cancel) {
            LCD_state = 3;  //set LCD to DOOR OPEN state 3
            latc.4 = 1;  //turn on buzzer
          }
          if(door_cancel) {  
            LCD_state = 0;  //set LCD to default (temps)
            latc.4 = 0;    //turn off buzzer
          }
          lata.2 = 1;  //turn on door LEDs
        }
        
               
          latc.3 = true_false.1;  //Icy Light
          lata.1 = true_false.2;  //Right Blind
          lata.0 = true_false.3;  //Left Blind
          lata.3 = true_false.5;  //Left Blink
          lata.5 = true_false.6;  //Right blink
         
                
        if(LCD_state == 5) { //LCD in backup - exit out b/c not in backup anymore
          LCD_state = 0;
        }
       }
      }
      
            
      if(identifier == 0b00000001) { //backup identifier
        LCD_state = 5;  //put LCD in backup mode
        backup_range = data;
        //turn off all lights
        true_false = 0;
        latc.3 = true_false.1;  //Icy Light
        lata.1 = true_false.2;  //Right Blind
        lata.0 = true_false.3;  //Left Blind
        lata.3 = true_false.5;  //Left Blink
        lata.5 = true_false.6;  //Right blink
        lata.2 = true_false.0;  //Door
      }
    
      if(identifier == 0b00000010) {  //temp in identifier
        if(LCD_state == 5) { //LCD in backup - exit out b/c not in backup anymore
          LCD_state = 0;
        }
        
        if(data != 126)
          temp_in = data;
                
        if(temp_in <= temp_in_low_alarm) { 
          if(temp_alarm_armed == 1) {
            LCD_state = 4;
            latc.4 = 1;  //turn on buzzer        
          }
        } 
        
        if(temp_in >= temp_in_high_alarm) {
          if(temp_alarm_armed == 1) {
            LCD_state = 4;
            latc.4 = 1;  //turn on buzzer        
          }
        }            
        
        if(temp_in > temp_in_low_alarm)  { 
          if(temp_in < temp_in_high_alarm) {
            temp_alarm_armed = 1;
            if(LCD_state == 3) {
              latc.4 = 0;  //turn off buzzer
              LCD_state = 0;  //back to default LCD
            }
          }
        }
      }
      
      if(identifier == 0b00000011) { //temp out identifier
        if(LCD_state == 5) { //LCD in backup - exit out b/c not in backup anymore
          LCD_state = 0;
        }
        
        if(data != 126)
         temp_out = data;
      }
    } // closes the dealing with data section
   
    /* State Machine 
    * 0 = default - temps
    * 1 = Set High temp alarm threshold
    * 2 = Set Low temp alarm threshold
    * 3 = Door Alarm
    * 4 = Temp In Alarm
    * 5 = Backup      */
        
    if(LCD_state == 0) {  
      LCD_setpos(0,0);
      LCD_printf("Temp in:                      ");
      LCD_setpos(0,10);
      LCD_int(temp_in);
      LCD_setpos(0,15);
      LCD_printf(" F          ");
      
      LCD_setpos(1,0);
      LCD_printf("Temp out:                     ");
      LCD_setpos(1,10);
      LCD_int(temp_out);      
      LCD_setpos(1,15);
      LCD_printf(" F          ");    
      
      if(enter == 1) {
        LCD_state = 1;
        enter = 0;
        LCD_clear();
      }
      
      if(cancel == 1) {
        LCD_state = 0;
        cancel = 0;
      }
      
      if(up_arrow) {
        up_arrow = 0;
      }
      
      if(down_arrow) {
        down_arrow = 0;
      }
   }
   
   if(LCD_state == 1) {
     LCD_setpos(0,0);
     LCD_printf("High Temp Alarm");
     LCD_setpos(0,16);
     LCD_int(temp_in_high_alarm2);
     LCD_printf("F");
          
     if(enter == 1) {
       LCD_state = 2;
       temp_in_high_alarm = temp_in_high_alarm2;
       LCD_setpos(1,22);
       LCD_printf("OK");
       delay_s(1);
       LCD_clear();
       enter = 0;
     } 
     
     if(up_arrow == 1) {
       temp_in_high_alarm2 = temp_in_high_alarm2 + 1;
       up_arrow = 0;
     }
     
     if(down_arrow == 1) {
       temp_in_high_alarm2 = temp_in_high_alarm2 - 1;
       down_arrow = 0;
     }
     
     if(cancel == 1) {
       LCD_state = 0;
       cancel = 0;
       temp_in_high_alarm2 = temp_in_high_alarm;
       LCD_clear();
     } 
   }
   
   if(LCD_state == 2) {
     LCD_setpos(0,0);
     LCD_printf("Low Temp Alarm: ");
     LCD_setpos(0,16);
     LCD_int(temp_in_low_alarm2);
     LCD_printf("F");     
    
     if(enter == 1) {
       LCD_state = 0;
       temp_in_low_alarm = temp_in_low_alarm2;
       LCD_setpos(1,22);
       LCD_printf("OK");
       delay_s(1);
       LCD_clear();
       enter = 0;
     } 
     
     if(up_arrow == 1) {
       temp_in_low_alarm2 = temp_in_low_alarm2 + 1;
       up_arrow = 0;
     }
     
     if(down_arrow == 1) {
       temp_in_low_alarm2 = temp_in_low_alarm2 - 1;
       down_arrow = 0;
     }
     
     if(cancel == 1) {
       LCD_state = 0;
       cancel = 0;
       temp_in_low_alarm2 = temp_in_low_alarm;
       LCD_clear();
     } 
   }

  if(LCD_state == 3) {
    LCD_setpos(0,0);
    LCD_printf("       DOOR OPEN                  ");
    LCD_setpos(1,0);
    LCD_printf("                                  ");
    
    if(enter == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       door_cancel = true;
       LCD_clear();
       enter = 0;
    } 
     
    if(up_arrow == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       door_cancel = true;
       LCD_clear();
       up_arrow = 0;
    }
     
    if(down_arrow == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       door_cancel = true;
       LCD_clear();
       down_arrow = 0;
    }
     
    if(cancel == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       door_cancel = true;
       LCD_clear();
       cancel = 0;
    }    
  }
  
  if(LCD_state == 4) {
    LCD_setpos(0,0);
    LCD_printf("     Temp In Alarm");
    LCD_setpos(1,0);
    LCD_printf("Range    -     Temp");
    LCD_setpos(1,6);
    LCD_int(temp_in_low_alarm);
    LCD_setpos(1,10);
    LCD_int(temp_in_high_alarm);
    LCD_setpos(1,20);
    LCD_dec(temp_in);
    
    
    if(enter == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       temp_alarm_armed = 0;
       LCD_clear();
       enter = 0;
    } 
     
    if(up_arrow == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       temp_alarm_armed = 0;
       LCD_clear();
       up_arrow = 0;
    }
     
    if(down_arrow == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       temp_alarm_armed = 0;
       LCD_clear();
       down_arrow = 0;
    }
     
    if(cancel == 1) {
       LCD_state = 0;
       latc.4 = 0;  //turn off buzzer
       temp_alarm_armed = 0;
       LCD_clear();
       cancel = 0;
    }    
  }

  if(LCD_state == 5) {
      
   inches = backup_range;
   LCD_setpos(1,0);
   LCD_printf("                        ");
      
    if(inches <= 8) {
      too_close = 1;
    }
    if(inches >= 18) {
      too_close = 0;
    }
    
    if(too_close) {
      LCD_clear();
      LCD_setpos(0,0);
      LCD_printf("Too Close!!!      ");
      lata.4 = 1;  //turn on red 'too close' leds
      delay_ms(200);
    }
 
    feet = inches/12;
    inches_after_feet = inches - 12*feet;
   
    if(!too_close && feet != 10) {
      lata.4 = 0;  //turn off red 'too close' leds
      LCD_setpos(0,7);
      LCD_printf("  ft   in ");
      LCD_setpos(0,0);
      LCD_printf("Range =  ");
      LCD_setpos(0,7);
      LCD_int(feet);
      LCD_setpos(0,9);
      LCD_printf("ft ");
      LCD_setpos(0,12);
      LCD_int(inches_after_feet);
      LCD_setpos(0,14);
      LCD_printf("in");
      delay_ms(300);
    }
    
    if(enter == 1) {
      enter = 0;
    }
      
    if(up_arrow == 1) {
      up_arrow = 0;
    }
    
    if(down_arrow == 1) {
      down_arrow = 0;
    }
      
    if(cancel == 1) {
      cancel = 0;
    }
  }        
   
   
}//closes the while loop
}//closes main

void interrupt(void) {
   
  if (intcon.0==1) { //interrupt for B4-B7
    
    if(portb.4==1) {  //up arrow not pushed
      up_arrow = false; 
      
    }   
    if(portb.4==0) {  //up arrow pushed
      up_arrow = true;
      
    }
    if(portb.5==1) {  //down arrow not pushed
      down_arrow = false;
      
    }
    if(portb.5==0) {  //down arrow pushed
      down_arrow = true;
    }
    if(portb.6==1) {  //enter not pushed
      enter = false;
     
    }
    if(portb.6==0) {  //enter pushed
      enter = true;
      
    }
    if(portb.7==1) {  //cancel not pushed
      cancel = false;
      
    }
    if(portb.7==0) {  //cancel pushed
      cancel = true;
    }
  intcon.0 = 0;  //clear interrupt
  }
  
  if(pir1.5==1) { //determining what each byte received is (start, identifier, or data)
	received = rcreg;
    if(byte_received == 2) {
      data = received;
      send_acknowledge = 1;
      loop_count = 0;
    }
    if(byte_received == 1) {
      identifier = received;
      byte_received = 2;
    }
	if(received == 0x7E) {
	  byte_received = 1;
	}
  }
}

