#include    <system.h>
#include    "EESD.h"

/*
 * This is a starting point of a library
 * of functions that are useful. THese
 * fuctions drive the LCD dispaly.
 *
 * It is recommended that you create a
 * similar set of fucntions that work with
 * HyperTerminal
 * */
  /*
   * Setup LCD display
   * */

void LCD_init(void)
{
  trisd = trisd & 0x0f;  //Top 4 pins configured for output
  trise = trise & 0xf8;  //Bottom 3 pins configured for output
  adcon1 |= 0x0f;             // port E digital mode
    
  delay_ms(40);

  LCD_icmd( 0x30 ) ;          /*  8 Bit mode  */
  delay_ms(5);			// DH - min delay here of 4.1 ms
  LCD_icmd( 0x30 ) ;          /*  8 Bit mode  */
  LCD_icmd( 0x30 ) ;          /*  8 Bit mode  */

  LCD_icmd( 0x20 ) ;          /*  4-BIT Mode  */
  LCD_cmd( 0x28 );            /*  Function Set: 4-bit,2-line,5X7 */

  LCD_cmd( 0x0C );            /*  Display on, Cursor off         */
  LCD_cmd( 0x06 );            /*  Entry mode: INC addr, NO SHIFT display */

  LCD_cmd( 0x01 );            /*  Clear Display */
  delay_ms( 20 );
}


/*******************************************************
 *  LCD_cmd - Write ASCII character to LCD peripheral  *
 *      configured as a 4 bit interface                *
 *******************************************************/
void LCD_char(char data)
{
  char dsave;
  dsave = latd;
  LCD_DATA = (data & 0xF0) | (LCD_DATA & 0x0F);         /* Write HI nibble word to LCD */
  write_data;
  LCD_DATA = ((data << 4) & 0xF0) | (LCD_DATA & 0x0F);  /* Write LO nibble word to LCD */
  write_data;
  delay_ms(1);
  latd = dsave;
}
/*****************************************************
 *  LCD_icmd - Write control word to LCD peripheral  *
 *****************************************************/
void LCD_icmd(char data)
{
  char dsave;
  dsave = portd;
  LCD_DATA = data | (LCD_DATA & 0x0F) ;
  write_cmd;

  delay_ms(1);
  portd = dsave;
  return;
}

/***************************************************
 *  LCD_cmd - Write control word to LCD peripheral *
 *      configured as a 4 bit interface            *
 **************************************************/
void LCD_cmd(char data)
{
  char dsave;
  dsave = portd;
  LCD_DATA = (data & 0xF0) | (LCD_DATA & 0x0F);         /* Write HI nibble word to LCD */
  write_cmd;
  LCD_DATA = ((data << 4) & 0xF0) | (LCD_DATA & 0x0F);  /* Write LO nibble word to LCD */
  write_cmd;
  if (data < 0x03) delay_ms(20); else delay_ms(1);
  portd = dsave;
  return;

}
/* set the cursor position */

void LCD_setpos(char line, char pos)
{
  LCD_cmd(0x80 + line * 0x40 + pos);
  return;
}

/**********************
 * routine to clear display screen
 * */
 
void LCD_clear(void)
{
  LCD_cmd( 0x01 );            /*  Clear Display */
  delay_ms(20);
  return;
}

/********************************
 * Display char in HEX - format *
 ********************************/
void LCD_hex(char data)
{
  char n;
  n = ((data >> 4) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}
/********************************
 * Display unsigned short in HEX - format *
 ********************************/
void LCD_hex(unsigned short	  data)
{
  char n;
  n = ((data >> 12) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 8) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 4) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}
/********************************
 * Display  short in HEX - format *
 ********************************/
void LCD_hex(short data)
{
  char n;
  n = ((data >> 12) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 8) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 4) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}
/********************************
 * Display low nibble in HEX - format *
 ********************************/
void LCD_nib(char data)
{
  char n;
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}

/********************************
 * Display byte in binary - format *
 ********************************/
void LCD_bin(char data)
{
  char n;
  char temp;
  temp =0x80;
  for( n=1; n<=8; ++n)
  {
    if(temp & data)
      LCD_char(0x31);
    else
      LCD_char(0x30);
    temp = temp >> 1;
  }
  return;
}
/********************************
 * Display unsigned short in binary - format *
 ********************************/
void LCD_bin(unsigned short data)
{
  char n;
  unsigned short temp;
  temp =0x8000;
  for( n=1; n<=16; ++n)
  {
    if(temp & data)
      LCD_char(0x31);
    else
      LCD_char(0x30);
    temp = temp >> 1;
  }
}

void LCD_dec(short dat)
{
  unsigned short val;   // ascii results
  unsigned short temp;
  unsigned short div;
  unsigned short data;
  char i;
  char digit;
  data = dat;  // make it unsigned
  div = 10000;
  for(i=0; i <= 4; ++i)   // get all 5 digits
  {
    val = data/div;       // get most signif. digit
    LCD_char(val + '0');    // print digit
    data -= val * div;   // what weve printed
    div=div/10;         // adjust divisor
  }
  return;
}

void LCD_dec(unsigned short dat)
{
  unsigned short val;   // ascii results
  unsigned short temp;
  unsigned short div;
  unsigned short data;
  char i;
  char digit;
  data = dat;  // make it unsigned
  div = 10000;
  for(i=0; i <= 4; ++i)   // get all 5 digits
  {
    val = data/div;       // get most signif. digit
    LCD_char(val + '0');    // print digit
    data -= val * div;   // what weve printed
    div=div/10;         // adjust divisor
  }
  return;
}
/**********************************
 * Display byte as decimal number *
 **********************************/
void LCD_dec(char data)
{
  LCD_char(((data /100) & 255) + 0x30);
  data = data % 100;
  LCD_char( ((data / 10) & 255) + 0x30 );
  LCD_char( ((data % 10) & 255) + 0x30 );
  return;
}
/*
 * display char as a signed int
 * */

void LCD_int(char data)
{
  char val;   // ascii results
  char temp;
  char div;
  char i;
  bool dozero;
  dozero = false;
  div = 100;
  if(data == 0)  // always print 0
  {
    LCD_char('0');
    return;
  }
    /*
     * adjust for sign
     * */
  if(data & 10000000b)
  {
    LCD_char('-');
    data = ~data + 1;
  }
  for(i=0; i <= 2; ++i)   // get all 3 digits
  {
    val = data/div;       // get most signif. digit
    if(val != 0 || dozero)
    { 
      LCD_char(val + '0');    // print digit
      dozero = true;
    }
    temp = val * div;   // subtract off
    data = data - temp;   // what weve printed
    div=div/10;         // adjust divisor
  }
}
/*
 * display unsigned short as a signed int
 * */

void LCD_int(short dat)
{
  short val;   // ascii results
  short temp;
  short div;
  char i;
  unsigned short data;
  bool dozero;
  dozero = false;
  data = dat;
  if(dat == 0)  // always print 0
  {
    LCD_char('0');
    return;
  }
  div = 10000;
    /*
     * adjust for sign
     * */
  if(dat < 0)
  {
    LCD_char('-');
    data = ~data + 1;
  }
  for(i=0; i <= 4; ++i)   // get all 3 digits
  {
    val = data/div;       // get most signif. digit
    if(val != 0 || dozero) 
    {
      LCD_char(val + '0');    // print digit
      dozero = true;
    }
    data -= val*div;   // subtract off what weve printed
    div=div/10;         // adjust divisor
  }
}

void LCD_printf( const char* text )
{
  char i = 0;
  while( text[i] != 0 )
    LCD_char( text[i++] );
  return;
}



void IR_init_left(void) {
  adcon0 = 0b00001101;
  adcon1 = 0b00001011;
  //trisa.3 = 1;  // AN3 configured as input
  adcon2 = 0b10111001;
}

void IR_init_right(void) {
  adcon0 = 0b00001001;
  adcon1 = 0b00001100;
  //trisa.2 = 1;  // AN2 configured as input
  adcon2 = 0b10111001;
}

unsigned short IR_range_inch(void) {

	unsigned short result;
	short inches;
	
	result = adconvert();

    if(result >= 230) {
      inches = 28 + (33/100) - 4*result/100 - 1*result/1000 - 4*result/10000;   //convert to inches
    }
    if(result <= 229) {
      inches = 63 + 38/100 - 2*result/10 - 1*result/100 - 8*result/1000; //convert to inches
    }
    	
return(inches);
}

void putc(char value)
{
	
	while(!txsta.1) { }
	txreg = value;
	//LCD_printf("putc1");
	
}

void putcf( const char* text )
{
  char i = 0;
  while( text[i] != 0 ) {
    putc( text[i++] );
  }
  //putc(0x0D);
  return;
}


void zigbee_init(void){
	
	trisc.6=1;
	trisc.7=1;
	
    txsta = 0x24;
	rcsta = 0x90;
	
	baudcon.3=0;
	baudcon.0=0;
	
	spbrgh = 0;
	spbrg = 255;
	
    trise.2 = 0;
	late.2 = 1;
	late.2 = 0;
	delay_ms(10);
	late.2 = 1;
	delay_ms(500);
	
	delay_s(2);  //get Zigbee into command mode
    putcf("+++");
    delay_s(2);
    delay_ms(100);
        
    putcf("ATCE 0");
    while(!txsta) { }
    txreg = 13;
  
    delay_ms(60);
    putcf("ATA1 00");
    while(!txsta) { }
    txreg = 13;
   
    delay_ms(60);
    putcf("ATID 3332");
    while(!txsta) { }
    txreg = 13;
    
    delay_ms(60);
    putcf("ATCH 0C");
    while(!txsta) { }
    txreg = 13;

    delay_ms(60);
    putcf("ATAC");
    while(!txsta) { }
    txreg = 13;    
    
      
}

void send_char(char value) {
	while(!txsta.1) { }
	txreg = value;
}

void send_short(short value) {
    while(!txsta.1) { }
	txreg = value;	
}

unsigned short adconvert(void) {
  unsigned short result;
  adcon0.1 = 1;  //start conversion
  while(adcon0.1); //wait for conversion to end
  result = adresl;
  result += (adresh << 8);  //add in adresh register but shifted 8 bits to the left.
   
  return result;
  }

short right_blinker_shunt(void) {
  unsigned short high;
  unsigned short low;
  unsigned short diff;
  short on;
  
  adcon0 = 0b00101001;  // B1, AN10 configured as analog
  adcon1 = 0b00000100;
  trisb.1 = 1;  // B1, AN10 configured as input
  adcon2 = 0b10111001;
  
  high = adconvert();
    
  adcon0 = 0b00010001;  // A5, AN4 configured as analog  
  trisa.5 = 1;
  
  low = adconvert();
  
  diff = high - low;
  
  LCD_setpos(0,15);
  LCD_dec(high);
  LCD_setpos(1,15);
  LCD_dec(low);
  
  if(diff >= 5)  //Need a threshold
    on = 1;
  
  if(diff <= 4)
    on = 0;
    
  return on;
} 


short left_blinker_shunt(void) {
  unsigned short high;
  unsigned short low;
  unsigned short diff;
  short on;
  
  adcon0 = 0b00100101;  // B3, AN9 configured as analog
  adcon1 = 0b00000100;
  trisb.3 = 1;  // B3, AN9 configured as input
  adcon2 = 0b10111001;
  
  high = adconvert();
  
  adcon0 = 0b00000001;  // A0, AN0 configured as analog  
  trisa.0 = 1;
  
  low = adconvert();
  
  diff = high - low;
  
  LCD_setpos(0,15);
  LCD_dec(high);
  LCD_setpos(1,15);
  LCD_dec(low);
  
  if(diff >= 5)  //Need a threshold
    on = 1;
  
  if(diff <= 4)
    on = 0;
  
  return on;
} 

short backup_check(void) {
  unsigned short high;
  short on = 0;
  
  adcon0 = 0b00100001;  // B2, AN8 configured as analog
  adcon1 = 0b00000100;
  trisb.2 = 1;  // B2, AN8 configured as input
  adcon2 = 0b10111001;
  
  high = adconvert();

  if(high > 300) 
    on = 1;
      
  /* adcon0 = 0b00000101;  // A1, AN1 configured as analog  
  trisa.1 = 1;
  
  low = adconvert(); */
     
  return on;
}

short get_temp_in(void) {
  unsigned short data;
  short tempf;
  
  adcon0 = 0b00000101;  // A1, AN1 configured as analog  
  adcon1 = 0b00000100;
  trisa.1 = 1;
  adcon2 = 0b10111001;
  
  data = adconvert(); 
  tempf = 8*data/10 + 7*data/100 + 8*data/1000 - 58;
  
  LCD_setpos(0,10);
  LCD_dec(data);
  LCD_setpos(1,10);
  LCD_dec(tempf);
    
  return tempf;
  
}

short brake_shunt(void) {
  unsigned short high;
  unsigned short low;
  unsigned short diff;
  short on;
  
  adcon0 = 0b00100001;  // B2, AN8 configured as analog
  adcon1 = 0b00000100;
  trisb.2 = 1;  // B2, AN8 configured as input
  adcon2 = 0b10111001;
  
  high = adconvert();
  
  adcon0 = 0b00000101;  // A1, AN1 configured as analog  
  adcon1 = 0b00000100;
  trisa.1 = 1;
  adcon2 = 0b10111001;
    
  low = adconvert();
  
  diff = high - low;
  
  if(diff >= 5)  //Need a threshold
    on = 1;
  
  if(diff <= 4)
    on = 0;
  
  return on;
} 


  
